syntax = "proto3";
package plugin;
option go_package = "plugin";

message Empty {
}

// The GRPCController is responsible for telling the plugin server to shutdown.
service GRPCController {
    rpc Shutdown(Empty) returns (Empty);
}
